-------------------------------------------------------------------------------
-- Name class  ApplicationEvents
-- Author Jonathan Primeau
--
-- Description
--     All the application events should be defined here
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent
property pAppEvents


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters
--     me (object) this pointer.
--
-- Return
--     me (object) A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class Events").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters
--     me (object) this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  me._UnregisterApplicationEvents()
  
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters
--     me (object) this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pAppEvents = []
  me.pAppEvents.add(#onTimeout)
  me.pAppEvents.add(#onPulse)
  me.pAppEvents.add(#onEnterbetActivity)
  -- Add new application events here
  
  me._RegisterApplicationEvents()
end

on _RegisterApplicationEvents (me)
  lCount = me.pAppEvents.count
  repeat with i = 1 to lCount
    me.RegisterEvent(me.pAppEvents[i])  
  end repeat
end

on _UnregisterApplicationEvents (me)
  lCount = me.pAppEvents.count
  repeat with i = 1 to lCount
    me.UnregisterEvent(me.pAppEvents[i])  
  end repeat 
end
