-------------------------------------------------------------------------------
-- Name: class FatalError
-- Author: Jonathan Primeau
--
-- Description:
--     Displays a fatal error message for a while
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent
property pWindow               -- Window object


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class Timeout").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  me.pWindow.CloseWindow()
  me.pWindow = void
  
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on Start (me)
  GenLog(#debug, "FatalError::Start: Starting timer...")
  
  me.ResetHit()
  me.SetName("to,fatalError")
  me.SetHitScale(1000) -- 1 second
  
  me.SetTimeout(5000) -- 5 seconds
  
  me.SetHitCallback(#_HitCallback, me)
  me.SetTimeoutCallback(#_TimeoutCallback, me)
  
  if (ancestor.Start() = true) then
    GenLog(#debug, "FatalError::Start: Timer started!")
  else 
    GenLog(#error, "FatalError::Start: Timer NOT started!")
  end if
end

on Stop (me)
  GenLog(#debug, "FatalError::Stop: Stopping timer...")
  ancestor.Stop()
  GenLog(#debug, "FatalError::Stop: Timer stopped!")
end

on Show (me)
  -- Show only if we are in run mode
  if (QI(#init).IsInitialized() = true and QI(#init).IsLoaded() = true) then
    --if (me.pWindow.IsVisible() = false) then
    GenLog(#debug, "FatalError::Show: Showing the fatal error...")
    
    me.pWindow.CenterWindow()
    lFatalErrorMsg = QI(#config).GetFatalErrorMessage()
    me.pWindow.GetMovieRef().InitDialog(me, "OK", lFatalErrorMsg)
    me.pWindow.ShowWindow()
    
    -- Start the timer
    me.Start()
    
    GenLog(#debug, "FatalError::Show: Timeout visible!")
    --end if
  end if
end

on Hide (me, theButton)
  GenLog(#debug, "FatalError::Hide: Hiding the timeout...")
  
  -- Stop the timer
  me.Stop()
  
  me.pWindow.HideWindow()
  
  GenLog(#debug, "FatalError::Hide: Timeout hidden!")
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pWindow = script("class Window").new()
  member("msgdialog,message").text = ""
  
  me.pWindow.OpenWindow("msgdialog", #dialog)
end

on _HitCallback (me)
  
end

on _TimeoutCallback (me)
  me.Hide()
end