-------------------------------------------------------------------------------
-- Name: class OPSCommunicator
-- Author: Jonathan Primeau
--
-- Description:
--     This is the layer between MUSClient and high level calls.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pCommand
property pLocalIP
property pCallback
property pFlashConnector
property pCommandRefList


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class MUSClient2").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
  
  me.pCommand = 0
end

on Connect (me, theHandler, theObject)
  me.pCallback.handler = theHandler
  me.pCallback.object = theObject
  
  lOPSInfo = QI(#config).GetOPSInfo()
  ancestor.SetIP(lOPSInfo.ip)
  ancestor.SetPort(lOPSInfo.port)
  
  ancestor.SetConnectionCallback(#_ConnectionHandler, me)
  ancestor.SetMessageCallback(#_MessageHandler, me)
  
  ancestor.Connect()
end

on IsConnected (me)
  return(ancestor.IsConnected())
end

on Disconnect (me)
  ancestor.Disconnect()
end

on InitializeKiosk (me, theHandler, theObject)
  me.pCallback.handler = theHandler
  me.pCallback.object = theObject
  
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #response, theHandler, theObject)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("InitializeKiosk")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("ip", me.pLocalIP)
  me.pCommand.AddAttribute("name", QI(#status).GetKioskName())
  me.pCommand.AddAttribute("version", QI(#config).GetKioskVersion())
  me.pCommand.AddAttribute("port", "50001")
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

on Status (me, theRefID)
  lStatus = QI(#status).GetStatus()
  
  lLogin = string(lStatus.login)
  lSequence = string(lStatus.sequence)
  lAdmission = string(lStatus.admission)
  lState = string(lStatus.state)
  lArea = string(lStatus.area)
  
  lXML = "<Status"
  lXML = lXML && "login=" &QUOTE& lLogin &QUOTE
  lXML = lXML && "sequence=" &QUOTE& lSequence &QUOTE
  lXML = lXML && "admission=" &QUOTE& lAdmission &QUOTE
  lXML = lXML && "state=" &QUOTE& lState &QUOTE
  lXML = lXML && "area=" &QUOTE& lArea &QUOTE
  lXML = lXML & "/>"
  
  me.pCommand.BeginCommand()
  me.pCommand.SetCodeStatus("100", "Complete")
  me.pCommand.AddAttribute("refid", theRefID)
  me.pCommand.AddAttribute("data", me.pCommand.GetDataLength(lXML))
  me.pCommand.EndControl()
  me.pCommand.SetData(lXML)
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
end

on FetchSkinName (me, theHandler, theObject)
  me.pCallback.handler = theHandler
  me.pCallback.object = theObject
  
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #response, theHandler, theObject)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("FetchSkinName")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("kiosk", QI(#status).GetKioskID())
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

on FetchUrlList (me, theHandler, theObject)
  me.pCallback.handler = theHandler
  me.pCallback.object = theObject
  
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #response, theHandler, theObject)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("FetchUrlList")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("kiosk", QI(#status).GetKioskID())
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

on FakeFetchUrlList (me)
  lData = member("testurl").text
  QI(#internet).SetInternetXML(lData)
  QI(#internet).Load()
end

on FetchTvChannelList (me, theHandler, theObject)
  me.pCallback.handler = theHandler
  me.pCallback.object = theObject
  
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #response, theHandler, theObject)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("FetchTvChannelList")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("kiosk", QI(#status).GetKioskID())
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

on FakeFetchTvChannelList (me)
  lData = member("testvideo").text
  QI(#video).SetVideoXML(lData)
  QI(#video).Load()
end

on FetchBannerList (me, theHandler, theObject)
  me.pCallback.handler = theHandler
  me.pCallback.object = theObject
  
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #response, theHandler, theObject)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("FetchBannerList")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("kiosk", QI(#status).GetKioskID())
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

on FetchAttractLoopName (me, theHandler, theObject)
  me.pCallback.handler = theHandler
  me.pCallback.object = theObject
  
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #response, theHandler, theObject)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("FetchAttractLoopName")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("kiosk", QI(#status).GetKioskID())
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

on FakeFetchWebProxyList (me, theURL)
  lData = member("testproxy").text
  QI(#webProxy).SetWebProxyXML(lData)
  QI(#webProxy).Load()
end

on FetchWebProxyList (me, theHandler, theObject)
  me.pCallback.handler = theHandler
  me.pCallback.object = theObject
  
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #response, theHandler, theObject)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("FetchWebProxyList")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("kiosk", QI(#status).GetKioskID())
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

on TryURL (me, theURL, theHandler, theObject)
  me.pCallback.handler = theHandler
  me.pCallback.object = theObject
  
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #response, theHandler, theObject)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("TryURL")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("kiosk", QI(#status).GetKioskID())
  me.pCommand.AddAttribute("url", theURL)
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

on FillCrmOffer (me, theType, theHandler, theObject)
  lPatronID = 0
  lEvent = ""
  lRace = ""
  lLocation = QI(#status).GetLocation()
  
  -- Get the patron ID
  lPatronInfo = QI(#patron).GetPatronInfo()
  if (lPatronInfo.getaProp(#id) <> void) then
    lPlayerID = lPatronInfo.player
  end if
  
  if (theType = "QuickBets") then
    lQuickBetsInfo = QI(#patron).GetQuickBetsInfo()
    
    -- Get event
    if (lQuickBetsInfo.getaProp(#event) <> void) then
      lEvent = lQuickBetsInfo.event
    end if
    
    -- Get race
    if (lQuickBetsInfo.getaProp(#race) <> void) then
      lRace = lQuickBetsInfo.race
    end if
  end if
  
  -- Ignore QuickBets with event = 0 and race = 1
  if (lEvent = "0" and lRace = "1") then
    -- Don't send this to the server
    return(0)
  end if
  
  me.pCallback.handler = theHandler
  me.pCallback.object = theObject
  
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #response, theHandler, theObject)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("FillCrmOffer")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("kiosk", QI(#status).GetKioskID())
  me.pCommand.AddAttribute("clientID", lPlayerID)
  me.pCommand.AddAttribute("type", theType)
  me.pCommand.AddAttribute("location", lLocation)
  if (theType = "QuickBets") then
    me.pCommand.AddAttribute("event", lEvent)
    me.pCommand.AddAttribute("race", lRace)
  end if
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

on FillCrmOfferWager (me, theData, theHandler, theObject)
  me.pCallback.handler = theHandler
  me.pCallback.object = theObject
  
  lPatronID = 0
  lEvent = ""
  lRace = ""
  lLocation = QI(#status).GetLocation()
  
  -- Get the patron ID
  lPatronInfo = QI(#patron).GetPatronInfo()
  if (lPatronInfo.getaProp(#id) <> void) then
    lPlayerID = lPatronInfo.player
  end if
  
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #ignore, theHandler, theObject)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("FillCrmOffer")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("kiosk", QI(#status).GetKioskID())
  me.pCommand.AddAttribute("clientID", lPlayerID)
  me.pCommand.AddAttribute("type", "wager")
  me.pCommand.AddAttribute("location", lLocation)
  me.pCommand.AddAttribute("data", me.pCommand.GetDataLength(theData))
  me.pCommand.EndControl()
  me.pCommand.SetData(theData)
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

on ConfirmCrmOffer (me)
  lPatronID = 0
  lName = "Unknown"
  lType = string(QI(#crm).GetCrmType())
  
  lCrmInfo = QI(#crm).GetCrmInfo()
  
  lPatronInfo = QI(#patron).GetPatronInfo()
  
  if (lCrmInfo.getaProp(#offerName) <> void) then
    lName = lCrmInfo.offerName
  end if
  if (lPatronInfo.getaProp(#id) <> void) then
    lPlayerID = lPatronInfo.player
  end if
  
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #ignore)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("ConfirmCrmOffer")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("kiosk", QI(#status).GetKioskID())
  me.pCommand.AddAttribute("clientID", lPlayerID)
  me.pCommand.AddAttribute("type", lType)
  me.pCommand.AddAttribute("offerName", lName)
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

on LogActivity (me, theData)
  
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #ignore)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("LogActivity")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("kiosk", QI(#status).GetKioskID())
  me.pCommand.AddAttribute("data", me.pCommand.GetDataLength(theData))
  me.pCommand.EndControl()
  me.pCommand.SetData(theData)
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

on RaiseError (me, theData)
  
  if (QI(#init).IsInitialized() = true and QI(#init).IsLoaded() = true) then
    QI(#status).SetStatus(#state, "Error")
  else
    QI(#status).SetStatus(#state, "Critical")
  end if
  
  lTime = GetCurrentTime()
  lData = lTime && theData
  
  lXML = "<error><![CDATA[" & lData & "]]></error>"
  
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #ignore)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("RaiseError")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("kiosk", QI(#status).GetKioskID())
  me.pCommand.AddAttribute("data", me.pCommand.GetDataLength(lXML))
  me.pCommand.EndControl()
  me.pCommand.SetData(lXML)
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

on Acknowledge (me)
  lCommandID = me.pCommand.GetID()
  me._ProduceCommandRef(lCommandID, #ignore)
  
  me.pCommand.BeginCommand()
  me.pCommand.SetOperation("200 Complete")
  me.pCommand.AddAttribute("id", lCommandID)
  me.pCommand.AddAttribute("kiosk", QI(#status).GetKioskID())
  me.pCommand.EndCommand()
  
  ancestor.SendCommand(me.pCommand.Get())
  
  return(lCommandID)
end

-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pCommand = script("class Command").new()
  me.pCallback = [#handler: void, #object: void]
  me.pLocalIP = ancestor.GetIPAddress()
  me.pFlashConnector = sprite("Terminal")
  
  me.pCommandRefList = []
end

on _ConnectionHandler (me)
  -- Connection is made to the OPS
  call(me.pCallback.handler, me.pCallback.object)
end

on _MessageHandler (me, theContent)
  -- We received a message from the OPS, parse it.
  
  lControlData = me._SplitControlData(theContent)
  
  lControl = lControlData.control
  lData = lControlData.data
  
  GenLog(#info, "MUSClient::_MessageHandler: Control:" && lControl)
  GenLog(#info, "MUSClient::_MessageHandler: Data:" && lData)
  
  if (lControl starts "100" or lControl starts "200" or lControl starts "400") then
    -- We got something or there is a "soft" error
    lControl = me._SplitAnwserControl(lControl)
    lRefID = lControl.refID
    
    -- Get command information
    lCmdInfo = me._ConsumeCommandRef(lRefID)
    lAction = lCmdInfo.action
    lHandler = lCmdInfo.handler
    lObject = lCmdInfo.object
    
    if (lAction <> #ignore and lHandler <> void and lObject <> void) then
      call(lHandler, lObject, lControl, lData)
    end if
    
  else if (lControl starts "500") then
    -- Fatal error
    lControl = me._SplitAnwserControl(lControl)
    lRefID = lControl.refID
    lCmdInfo = me._ConsumeCommandRef(lRefID)
    
    if (QI(#init).IsInitialized() = false or QI(#init).IsLoaded() = false) then
      -- Will reconnect in a while
      QI(#opsCom).Disconnect()
    end if
    
  else if (lControl starts "Status") then
    
    -- No need to acknowledge since we answer directly to the command
    lControl = me._SplitRequestControl(lControl)
    me.Status(lControl.refid)
    
  else if (lControl starts "SetConfiguration") then
    
    me.Acknowledge()
    lControl = me._SplitRequestControl(lControl)
    QI(#config).SetConfiguration(lData)
    
  else if (lControl starts "Navigate") then
    
    me.Acknowledge()
    lControl = me._SplitRequestControl(lControl)
    QI(#nav).ServerNavigate(lData)
    
  else if (lControl starts "SetKioskState") then
    
    me.Acknowledge()
    lControl = me._SplitRequestControl(lControl)
    QI(#status).ServerSetStatus(lData)
    
  else if (lControl starts "UrlListAvailable") then
    
    me.Acknowledge()
    me.FetchUrlList(#_NewUrlList, me)
    
  else if (lControl starts "TvChannelListAvailable") then
    
    me.Acknowledge()
    me.FetchTvChannelList(#_NewTvChannelList, me)
    
  else if (lControl starts "BannerListAvailable") then
    
    me.Acknowledge()
    me.FetchBannerList(#_NewBannerList, me)
    
  else if (lControl starts "WebProxyListAvailable") then
    
    me.Acknowledge()
    me.FetchWebProxyList(#_NewWebProxyList, me)
    
  else if (lControl starts "ApplySkin") then
    
    me.Acknowledge()
    
    ActLog(#operational, "ApplySkin", "New skin applied by the operator")
    QI(#terminalCom).EndPatron()
    QI(#opsCom).Disconnect()
    QI(#init).ConnectionTimeout()
    
  else if (lControl starts "Enable") then
    
    me.Acknowledge()
    
    ActLog(#operational, "Enable", "Kiosk enabled by the operator")
    QI(#terminalCom).EndPatron()
    QI(#opsCom).Disconnect()
    QI(#init).ConnectionTimeout()
    
  else if (lControl starts "Disable") then
    
    me.Acknowledge()
    
    ActLog(#operational, "Disable", "Kiosk disabled by the operator")
    QI(#terminalCom).EndPatron()
    QI(#opsCom).Disconnect()
    QI(#init).ConnectionTimeout()
    
  else if (lControl starts "Reinitialize") then
    
    me.Acknowledge()
    
    ActLog(#operational, "Reinitialize", "Kiosk reinitialized by the operator")
    QI(#terminalCom).EndPatron()
    QI(#opsCom).Disconnect()
    QI(#init).ConnectionTimeout()
    
  end if
end

on _InvalidData (me)
  QI(#status).SetStatus(#state, "Error")
end

on _NewUrlList (me, theControl, theData)
  if (listP(theControl)) then
    if (theControl.code = "100" or theControl.code = "200") then
      if (theData <> "") then
        -- Store the URL list
        QI(#internet).SetInternetXML(theData)
        
        me.Acknowledge()
        
        -- Load the new URL list
        if (QI(#internet).Load() <> 0) then
          ActLog(#content, "URL_List", "ERROR: Bad URL list received")
          me._InvalidData()
        else
          ActLog(#content, "URL_List", "New URL list received")
        end if
      end if
    end if
  end if
end

on _NewTvChannelList (me, theControl, theData)
  if (listP(theControl)) then
    if (theControl.code = "100" or theControl.code = "200") then
      if (theData <> "") then
        -- Store the URL list
        QI(#video).SetVideoXML(theData)
        
        me.Acknowledge()
        
        -- Load the new channel list
        if (QI(#video).Load() <> 0) then
          ActLog(#content, "Channel_List", "ERROR: Bad channel list received")
          me._InvalidData()
        else
          ActLog(#content, "Channel_List", "New channel list received")
        end if
      end if
    end if
  end if
end

on _NewBannerList (me, theControl, theData)
  if (listP(theControl)) then
    if (theControl.code = "100" or theControl.code = "200") then
      if (theData <> "") then
        -- Store the URL list
        QI(#banner).SetBannerXML(theData)
        
        me.Acknowledge()
        
        -- Load the new banner list
        if (QI(#banner).Load() <> 0) then
          ActLog(#content, "Banner_List", "ERROR: Bad banner list received")
          me._InvalidData()
        else
          ActLog(#content, "Banner_List", "New banner list received")
          QI(#banner).Reset()
        end if
      end if
    end if
  end if
end

on _NewWebProxyList (me, theControl, theData)
  if (listP(theControl)) then
    if (theControl.code = "100" or theControl.code = "200") then
      if (theData <> "") then
        -- Store the URL list
        QI(#webProxy).SetWebProxyXML(theData)
        
        me.Acknowledge()
        
        -- Load the new banner list
        if (QI(#webProxy).Load() <> 0) then
          ActLog(#content, "WebProxy_List", "ERROR: Bad WebProxy list received")
          me._InvalidData()
        else
          ActLog(#content, "WebProxy_List", "New WebProxy list received")
        end if
      end if
    end if
  end if
end

on _SplitControlData (me, theStr)
  lStr = theStr
  
  -- Set the item delimiter
  lOldDelimiter = _player.itemDelimiter
  _player.itemDelimiter = RETURN
  
  -- Get the numbe of line in the string
  lLineCount = lStr.item.count
  
  lControl = ""
  if (lLineCount > 0) then lControl = lStr.item[1]
  
  lData = ""
  if (lLineCount > 1) then lData = lStr.item[2..lLineCount]
  
  lControlData = [:]
  lControlData.addProp(#control, lControl)
  lControlData.addProp(#data, lData)
  
  _player.itemDelimiter = lOldDelimiter
  
  return(lControlData)
end

on _SplitAnwserControl (me, theStr)
  -- 100 Complete refid: kiosk_001-104506-1, data: 24
  
  -- Set the item delimiter
  lOldDelimiter = _player.itemDelimiter
  _player.itemDelimiter = " "
  
  -- Code
  lCode = theStr.item[1]
  
  -- Status
  lStatus = theStr.item[2]
  
  -- RefID
  lRefID = theStr.item[4]
  -- Remove the "," at the end (if any)
  lCharCount = lRefID.char.count
  if (lRefID.char[lCharCount] = ",") then
    lRefID = lRefID.char[1..lCharCount-1]
  end if
  
  -- Data length
  lDataLength = theStr.item[6]
  
  lControl = [:]
  lControl.addProp(#code, lCode)
  lControl.addProp(#status, lStatus)
  lControl.addProp(#refid, lRefID)
  lControl.addProp(#length, lDataLength)
  
  _player.itemDelimiter = lOldDelimiter
  
  return(lControl)
end

on _SplitRequestControl (me, theStr)
  -- Status id: ops1-152014-3
  
  -- Set the item delimiter
  lOldDelimiter = _player.itemDelimiter
  _player.itemDelimiter = " "
  
  -- Request
  lRequest = theStr.item[1]
  
  -- RefID
  lRefID = theStr.item[3]
  
  lControl = [:]
  lControl.addProp(#request, lRequest)
  lControl.addProp(#refid, lRefID)
  
  _player.itemDelimiter = lOldDelimiter
  
  return(lControl)
end

on _ProduceCommandRef (me, theCommandID, theAction, theHandler, theObject)
  lCommandRef = [:]
  lCommandRef.addProp(#commandID, theCommandID)
  lCommandRef.addProp(#action, theAction)
  lCommandRef.addProp(#handler, theHandler)
  lCommandRef.addProp(#object, theObject)
  GenLog(#debug, "OPSCommunicator::_ProduceCommandRef: Queueing" && lCommandRef)
  me.pCommandRefList.add(lCommandRef)
end

on _ConsumeCommandRef (me, theCommandID)
  lCount = me.pCommandRefList.count
  repeat with i = 1 to lCount
    lCommandRef = me.pCommandRefList[i]
    if (theCommandID = lCommandRef.commandID) then
      me.pCommandRefList.deleteAt(i)
      GenLog(#debug, "OPSCommunicator::_ConsumeCommandRef: Consuming" && lCommandRef)
      return(lCommandRef)
    end if
  end repeat
  GenLog(#debug, "OPSCommunicator::_ConsumeCommandRef: Consuming unfound command" && theCommandID)
  return([action: #ignore, #handler: void, #object: void])
end