-------------------------------------------------------------------------------
-- Name: class Timer
-- Author: Jonathan Primeau
--
-- Description:
--     This class is a timer
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pStart
property pStop
property pAverage
property pCount
property pSum

-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on Reset (me)
  me.pAverage = 0
  me.pCount = 0
  me.pSum = 0
end

on Start (me)
  me.pStart = the ticks
end

on Stop (me)
  me.pStop = the ticks
  
  lDeltaTicks = me.pStop - me.pStart 
  lMS = me._TicksToMS(lDeltaTicks)
  
  if (lMS > 0) then -- Use only ms > 0
    me.pCount = me.pCount + 1
    me.pSum = me.pSum + lMS
  end if
  
  if (me.pCount < 1) then me.pCount = 1
  
  lDivide = me.pCount
  if (lDivide = 0) then lDivide = 1
  me.pAverage = me.pSum / lDivide
  
  return(lMS)
end

on GetAVG (me)
  return(me.pAverage)
end

-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.Reset()
end

on _TicksToMS (me, theTicks)
  lSec = float(theTicks) / 60
  lMS = integer(lSec * 1000)
  return(lMS)
end 