-------------------------------------------------------------------------------
-- Name class Window
-- Author Jonathan Primeau
--
-- Description
--     Window manager
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent
property pName
property pIsVisible


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters
--     me (object) this pointer.
--
-- Return
--     me (object) A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters
--     me (object) this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on OpenWindow (me, theName, theType)
  me.pName = theName
  
  if (me.IsOpen() = false) then
    -- Make sure that the window is closed
    me.CloseWindow()
    
    -- Create a new window
    new window(me.pName)
    window(me.pName).filename = me.pName
    me.SetType(#dialog)
    me.pIsVisible = false
  end if
end

on CloseWindow (me)
  if (me.IsOpen()) then
    window(me.pName).close()
    window(me.pName).forget()
  end if
  me.pIsVisible = false
end

on ShowWindow (me)
  if (me.IsOpen()) then
    window(me.pName).open()
    me.pIsVisible = true
  end if
end

on HideWindow (me)
  if (me.IsOpen()) then
    window(me.pName).close()
  end if
  me.pIsVisible = false
end

on SetType (me, theType)
  window(me.pName).type = theType
end

on IsVisible (me)
  if (me.IsOpen()) then
    return(me.pIsVisible)
  end if
  return(false)
end

on IsOpen (me)
  lCount = _player.windowList.count
  repeat with i = 1 to lCount
    if (_player.windowList[i].name = me.pName) then
      return(true)
    end if
  end repeat
  return(false)
end

on GetRect (me)
  if (me.IsOpen()) then
    lRect = window(me.pName).rect
    return(lRect)
  end if
  return(rect(0,0,0,0))
end

on SetRect (me, theRect)
  if (me.IsOpen()) then
    window(me.pName).rect = theRect
  end if
end

on GetMovieRef (me)
  if (me.IsOpen()) then
    return(window(me.pName).movie)
  end if
  return(void)
end

on CenterWindow (me)
  if (me.IsOpen()) then
    lCenterStagePosH = ((_movie.stage.rect.right - _movie.stage.rect.left) / 2) + _movie.stage.rect.left
    lCenterStagePosV = ((_movie.stage.rect.bottom - _movie.stage.rect.top) / 2) + _movie.stage.rect.top
    
    lCenterWinPosH = (me.GetRect().right - me.GetRect().left) / 2
    lCenterWinPosV = (me.GetRect().bottom - me.GetRect().top) / 2
    
    lLeft = lCenterStagePosH - lCenterWinPosH
    lTop = lCenterStagePosV - lCenterWinPosV
    
    lRight = lLeft + me.GetRect().width
    lBottom = lTop + me.GetRect().height
    
    me.SetRect(rect(lLeft,lTop,lRight,lBottom))
  end if
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters
--     me (object) this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pName = ""
  me.pIsVisible = false
end
